"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const http_1 = require("http");
function getLatestNodeVersion(packageName) {
    const DEFAULT_VERSION = 'latest';
    return new Promise(resolve => {
        return http_1.get(`http://registry.npmjs.org/${packageName}`, res => {
            let rawData = '';
            res.on('data', chunk => (rawData += chunk));
            res.on('end', () => {
                try {
                    const response = JSON.parse(rawData);
                    const latestVersion = (response &&
                        response['dist-tags'] &&
                        response['dist-tags'].latest) ||
                        {};
                    resolve(buildPackage(response.name || packageName, latestVersion));
                }
                catch (e) {
                    resolve(buildPackage(packageName));
                }
            });
        }).on('error', () => resolve(buildPackage(packageName)));
    });
    function buildPackage(name, version = DEFAULT_VERSION) {
        return { name, version };
    }
}
exports.getLatestNodeVersion = getLatestNodeVersion;
//# sourceMappingURL=npmjs.js.map